//------------------------------------------------------------------------------
// CERN BE-CEM-EDL
// General Cores Library
// https://www.ohwr.org/projects/general-cores
//------------------------------------------------------------------------------
//
// unit name: CSimDrv_PERIPH_SPI
//
// author: Grzegorz Daniluk
//
// description: SV DIOT Peripheral Board SPI driver for testbenches
//
//------------------------------------------------------------------------------
// Copyright CERN 2021
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __SIMDRV_PERIPH_SPI_SVH
`define __SIMDRV_PERIPH_SPI_SVH 1

// Peripheral Board SPI op-codes
`define POP_ID    0
`define POP_RSTAT 1
`define POP_WRITE 2
`define POP_READ  3

`define P_ADR(x) ((x) & 'hffffff)

class CSimDrv_PERIPH_SPI;

  protected CSimDrv_WB_SPI spim;

  function new(CSimDrv_WB_SPI oc_spi);
    spim = oc_spi;
  endfunction

  task init();
    spim.init();
  endtask

  task get_id(output uint32_t id);
    uint32_t rval;

    spim.cs(1);
    // send opcode
    spim.txrx(`POP_ID << 24, 32, rval);
    // read dummy id
    spim.txrx('h0, 32, rval);
    spim.cs(0);

    id = rval;
  endtask

  task write(uint32_t adr, uint32_t data[], int size);
    uint32_t rval;

    //$display("Writing(%08x) %d words", `P_ADR(adr), data.size());
    spim.cs(1);
    // send opcode
    spim.txrx(`POP_WRITE << 24 | `P_ADR(adr), 32, rval);
    // write some data
    for (int i=0; i<size; i++) begin
      //$display("w: %08X", data[i]);
      spim.txrx(data[i], 32, rval);
    end
    spim.cs(0);
  endtask

  task read(uint32_t adr, int size, output uint32_t data[]);
    uint32_t rval;

    data = new[size];

    //$display("Reading(%08x) %d words", `P_ADR(adr), size);
    spim.cs(1);
    // send opcode
    spim.txrx(`POP_READ << 24 | `P_ADR(adr), 32, rval);
    for (int i=0; i<size; i++) begin
      spim.txrx('h00, 32, data[i]);
      //$display("r: %08X", data[i]);
    end

    spim.cs(0);
  endtask

endclass

`endif
