library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity top_sim is
end top_sim;

architecture behav of top_sim is
  signal clk_25m_i  : std_logic := '0';
  signal por_n_b    : std_logic := 'Z';
  signal button_i   : std_logic;

  signal dati      : std_logic_vector(7 downto 0);
  signal dato      : std_logic_vector(7 downto 0);
  signal adr      : std_logic_vector(9 downto 0);
  signal stb      : std_logic;
  signal wclk     : std_logic;
  signal we       : std_logic;
  signal cyc      : std_logic;
  signal ack      : std_logic;
  signal rst      : std_logic;

  signal rstin_o    : std_logic;
  signal nostat_o   : std_logic;
  signal var3_acc : std_logic;
  signal var3_rdy : std_logic;
  signal var2_acc_o : std_logic;
  signal var2_rdy_i : std_logic;
  signal var1_acc : std_logic;
  signal var1_rdy : std_logic;
  signal p3_lgth_o  : std_logic_vector(2 downto 0);
  signal diag_scl_b : std_logic;
  signal diag_sda_b : std_logic;
  signal s1_p_b     : std_logic_vector(15 downto 0);
  signal s1_n_b     : std_logic_vector(15 downto 0);
  signal s1_en_i    : std_logic;
  signal s2_p_b     : std_logic_vector(13 downto 0);
  signal s2_n_b     : std_logic_vector(13 downto 0);
  signal s2_en_i    : std_logic;
  signal s3_p_b     : std_logic_vector(13 downto 0);
  signal s3_n_b     : std_logic_vector(13 downto 0);
  signal s3_en_i    : std_logic;
  signal s4_p_b     : std_logic_vector(13 downto 0);
  signal s4_n_b     : std_logic_vector(13 downto 0);
  signal s4_en_i    : std_logic;
  signal s5_p_b     : std_logic_vector(13 downto 0);
  signal s5_n_b     : std_logic_vector(13 downto 0);
  signal s5_en_i    : std_logic;
  signal s7_p_b     : std_logic_vector(15 downto 0);
  signal s7_n_b     : std_logic_vector(15 downto 0);
  signal s7_en_i    : std_logic;
  signal leds_o     : std_logic_vector(5 downto 0);

  type sultochar_type is array (std_ulogic) of character;
  constant sultochar : sultochar_type := "UX01ZWLH-";

  function image (v : std_logic_vector) return string
  is
    variable res : string (1 to v'length);
    variable p : natural := 1;
  begin
    for i in v'range loop
      res (p) := sultochar (v (i));
      p := p + 1;
    end loop;
    return res;
  end image;

begin
  clk_25m_i <= not clk_25m_i after 20 ns;
  button_i <= '1', '0' after 80 ns;
  por_n_b <= 'H';

  process (leds_o)
  begin
    report "Leds: " & image (leds_o) severity note;
  end process;

  var3_rdy <= '1';

  diot_urv_top_1: entity work.diot_urv_top
    port map (
      clk_25m_i  => clk_25m_i,
      por_n_b    => por_n_b,
      button_i   => button_i,
      dat_i      => dati,
      dat_o      => dato,
      adr_o      => adr,
      stb_o      => stb,
      wclk_o     => wclk,
      we_o       => we,
      cyc_o      => cyc,
      ack_i      => ack,
      rst_o      => rst,
      rstin_o    => rstin_o,
      nostat_o   => nostat_o,
      var3_acc_o => var3_acc,
      var3_rdy_i => var3_rdy,
      var2_acc_o => var2_acc_o,
      var2_rdy_i => var2_rdy_i,
      var1_acc_o => var1_acc,
      var1_rdy_i => var1_rdy,
      p3_lgth_o  => p3_lgth_o,
      diag_scl_b => diag_scl_b,
      diag_sda_b => diag_sda_b,
      s1_p_b     => s1_p_b,
      s1_n_b     => s1_n_b,
      s1_en_i    => s1_en_i,
      s2_p_b     => s2_p_b,
      s2_n_b     => s2_n_b,
      s2_en_i    => s2_en_i,
      s3_p_b     => s3_p_b,
      s3_n_b     => s3_n_b,
      s3_en_i    => s3_en_i,
      s4_p_b     => s4_p_b,
      s4_n_b     => s4_n_b,
      s4_en_i    => s4_en_i,
      s5_p_b     => s5_p_b,
      s5_n_b     => s5_n_b,
      s5_en_i    => s5_en_i,
      s7_p_b     => s7_p_b,
      s7_n_b     => s7_n_b,
      s7_en_i    => s7_en_i,
      leds_o     => leds_o);

  process (wclk) is
    subtype t_byte is std_logic_vector (7 downto 0);
    type t_byte_arr is array (natural range <>) of t_byte;
    subtype t_cmd is t_byte_arr (0 to 71);
    type t_cmds is array (natural range <>) of t_cmd;
    constant cmds : t_cmds (0 to 3) :=
      (--  First command: read mbox in.
       0 => (x"02", x"4b", x"00", x"00",
             x"14", x"00", x"10", x"00",
             others => x"00"),
       --  Second command: load plc
       1 => (x"10", x"2f", x"00", x"00",        --  Load block
             x"00", x"20", x"10", x"00",        --  In plc RAM
             x"93", x"01", x"00", x"04",        --  plc_demo code
             x"17", x"01", x"01", x"00",
             x"13", x"01", x"c1", x"3f",
             x"ef", x"00", x"c0", x"00",
             x"ef", x"00", x"40", x"01",
             x"6f", x"f0", x"df", x"ff",
             x"b7", x"07", x"10", x"00",
             x"23", x"a8", x"07", x"04",
             x"67", x"80", x"00", x"00",
             x"37", x"07", x"10", x"00",
             x"83", x"27", x"07", x"03",
             x"93", x"f7", x"17", x"00",
             x"b3", x"07", x"f0", x"40",
             x"93", x"f7", x"e7", x"ff",
             x"23", x"28", x"f7", x"04",
             x"6f", x"f0", x"df", x"fe"),
       --  Third command: deassert plc reset
       2 => (x"01", x"4b", x"00", x"00",
             x"00", x"00", x"10", x"00",
             x"01", x"00", x"00", x"00",
             others => x"00"),
       --  Fourth command: read mbox
       3 => (x"01", x"4b", x"00", x"00",
             x"00", x"00", x"10", x"00",
             x"01", x"00", x"00", x"00",
             others => x"00"));
    variable idx : natural := 0;
    variable addr : natural;
    variable ans : t_byte_arr (0 to 7);
    variable var3_acc_d : std_logic;
  begin
    if rising_edge(wclk) then
      if rst = '1' then
        idx := 0;
        var1_rdy <= '1';
        var3_acc_d := '0';
      else
        if cyc = '1' and stb = '1' then
          addr := to_integer (unsigned (adr(6 downto 0)));
          report "got WB read from " & natural'image (addr);
          if we = '0' and adr (9 downto 7) = "000" then
            dati <= cmds(idx)(addr - 2);
            ack <= '1';
          elsif we = '1' and adr (9 downto 7) = "010" then
            ans (addr - 2) := dato;
            ack <= '1';
          end if;
        else
          ack <= '0';
        end if;

        if var3_acc = '0' and var3_acc_d = '1' then
          report "got an answer";
          var1_rdy <= '0';
          idx := idx + 1;
        else
          if idx <= cmds'high then
            var1_rdy <= '1';
          end if;
        end if;
        var3_acc_d := var3_acc;
      end if;
    end if;
  end process;
end;
