`ifndef __SIM_PERIPH_TEST_SVH
`define __SIM_PERIPH_TEST_SVH

`define TEST_OK  0
`define TEST_ERR 1

class CSim_Trans_Params;
  randc int adr;
  rand  uint32_t data[];

  constraint c {adr > 0; adr < 800;}

  function new(int size);
    data = new[size];
  endfunction
endclass

class CSim_Periph_Test;

  protected CSimDrv_PERIPH_SPI periph;
  
  function new(CSimDrv_PERIPH_SPI drv);
    periph = drv;
  endfunction

  task init();
    periph.init();
  endtask

  task test_id();
    uint32_t rval;
    periph.get_id(rval);

    $write("Test ID.............");

    rval = rval & 'hFFFF0000;
    if (rval == 'hCAFE0000)
      $display("OK");
    else
      $display("Error (%08x)", rval);
  endtask

  task read_write(int num);
    uint32_t data[];
    uint32_t r_data[];
    uint32_t adr;
    uint32_t size;
    CSim_Trans_Params tr;
    int status = `TEST_OK;

    //constraint c (adr >0 and adr < 1000);

    $display("Test read_write.....");

    for (int i=0; i<num; i++) begin
      tr = new($urandom_range(10,200));
      assert(tr.randomize())
      else $fatal(0, "CSim_Periph_Test::read_write() randomization failed");

      periph.write(tr.adr, tr.data, tr.data.size());
      periph.read(tr.adr, tr.data.size(), r_data);
      if (tr.data == r_data)
        $display("OK(adr:%08x, sz:%d ", tr.adr, tr.data.size());
      else begin
        status = `TEST_ERR;
        $display("Error(adr:%08x, sz:%d ", tr.adr, tr.data.size());
      end
    end
    $display();

  endtask

  task leds(logic led_outs[]);
    uint32_t data[];
    uint32_t adr;

    data = new[1];
    data[0] = 'h05;

    adr = 'h400;
    periph.write(adr, data, 1);
    //if (led_outs == 'b0101)
    //  $display("OOOOOKKKKK");
    //else
    //  $display("BBAAADDD");
    data[0] = 'h00;
    periph.write(adr, data, 1);
  endtask

endclass

`endif
